@@ -6,7 +6,7 @@ import logging
 import sys
 
 import pandas
-from pyezviz import EzvizCamera, EzvizClient
+from pyezviz import EzvizCamera, EzvizClient, MQTTClient
 from pyezviz.constants import DefenseModeType
 
 
@@ -42,6 +42,9 @@ def main():
     parser_home_defence_mode = subparsers.add_parser(
         "home_defence_mode", help="Set home defence mode"
     )
+
+    parser_mqtt = subparsers.add_parser("mqtt", help="Set home defence mode")
+
     parser_home_defence_mode.add_argument(
         "--mode", required=False, help="Choose mode", choices=["HOME_MODE", "AWAY_MODE"]
     )
@@ -97,7 +100,7 @@ def main():
     parser_camera_alarm.add_argument(
         "--sound",
         required=False,
-        help="Sound level (2 is disabled, 1 intensive, 0 software)",
+        help="Sound level (2 is silent, 1 intensive, 0 soft)",
         type=int,
         choices=[0, 1, 2],
     )
@@ -136,7 +139,7 @@ def main():
             try:
                 client.login()
                 print(json.dumps(client.get_device(), indent=2))
-            except BaseException as exp:
+            except Exception as exp:  # pylint: disable=broad-except
                 print(exp)
             finally:
                 client.close_session()
@@ -144,7 +147,6 @@ def main():
         elif args.device_action == "status":
             try:
                 client.login()
-                # print(json.dumps(client.load_cameras(), indent=2))
                 print(
                     pandas.DataFrame(client.load_cameras()).to_string(
                         columns=[
@@ -177,7 +179,7 @@ def main():
                         ]
                     )
                 )
-            except BaseException as exp:
+            except Exception as exp:  # pylint: disable=broad-except
                 print(exp)
             finally:
                 client.close_session()
@@ -186,7 +188,7 @@ def main():
             try:
                 client.login()
                 print(json.dumps(client.get_switch(), indent=2))
-            except BaseException as exp:
+            except Exception as exp:  # pylint: disable=broad-except
                 print(exp)
             finally:
                 client.close_session()
@@ -195,7 +197,7 @@ def main():
             try:
                 client.login()
                 print(json.dumps(client.get_connection(), indent=2))
-            except BaseException as exp:
+            except Exception as exp:  # pylint: disable=broad-except
                 print(exp)
             finally:
                 client.close_session()
@@ -216,11 +218,23 @@ def main():
                         indent=2,
                     )
                 )
-            except BaseException as exp:
+            except Exception as exp:  # pylint: disable=broad-except
                 print(exp)
             finally:
                 client.close_session()
 
+    elif args.action == "mqtt":
+
+        try:
+            token = client.login()
+            mqtt = MQTTClient(token)
+            mqtt.start()
+
+        except Exception as exp:  # pylint: disable=broad-except
+            print(exp)
+        finally:
+            client.close_session()
+
     elif args.action == "camera":
 
         # load camera object
@@ -228,14 +242,14 @@ def main():
             client.login()
             camera = EzvizCamera(client, args.serial)
             logging.debug("Camera loaded")
-        except BaseException as exp:
+        except Exception as exp:  # pylint: disable=broad-except
             print(exp)
             client.close_session()
 
         if args.camera_action == "move":
             try:
                 camera.move(args.direction, args.speed)
-            except BaseException as exp:
+            except Exception as exp:  # pylint: disable=broad-except
                 print(exp)
             finally:
                 client.close_session()
@@ -244,7 +258,7 @@ def main():
             try:
                 print(json.dumps(camera.status(), indent=2))
 
-            except BaseException as exp:
+            except Exception as exp:  # pylint: disable=broad-except
                 print(exp)
             finally:
                 client.close_session()
@@ -264,7 +278,7 @@ def main():
                     camera.switch_sleep_mode(args.enable)
                 elif args.switch == "follow_move":
                     camera.switch_follow_move(args.enable)
-            except BaseException as exp:
+            except Exception as exp:  # pylint: disable=broad-except
                 print(exp)
             finally:
                 client.close_session()
@@ -279,7 +293,7 @@ def main():
                     camera.alarm_detection_sensibility(args.sensibility)
                 if args.schedule is not None:
                     camera.change_defence_schedule(args.schedule)
-            except BaseException as exp:
+            except Exception as exp:  # pylint: disable=broad-except
                 print(exp)
             finally:
                 client.close_session()
