@@ -2,10 +2,7 @@
 import datetime
 
 from pyezviz.constants import DeviceCatagories, DeviceSwitchType, SoundMode
-
-
-class PyEzvizError(Exception):
-    """handle exception."""
+from pyezviz.exceptions import PyEzvizError
 
 
 class EzvizCamera:
@@ -75,13 +72,14 @@ class EzvizCamera:
 
     def _local_ip(self):
         """"Fix empty ip value for certain cameras"""
+        if self._device.get("wifiInfos"):
+            return self._device.get("wifiInfos").get("address")
+
+        # Seems to return none or 0.0.0.0 on some. This should run 2nd.
         if self._device.get("connectionInfos"):
             if self._device.get("connectionInfos").get("localIp"):
                 return self._device.get("connectionInfos").get("localIp")
 
-        if self._device.get("wifiInfos"):
-            return self._device.get("wifiInfos").get("address")
-
         return "0.0.0.0"
 
     def _motion_trigger(self):
@@ -190,7 +188,7 @@ class EzvizCamera:
 
     def alarm_notify(self, enable):
         """Enable/Disable camera notification when movement is detected."""
-        return self._client.data_report(self._serial, enable)
+        return self._client.set_camera_defence(self._serial, enable)
 
     def alarm_sound(self, sound_type):
         """Enable/Disable camera sound when movement is detected."""
