@@ -5,7 +5,10 @@ from uuid import uuid4
 
 import requests
 from pyezviz.camera import EzvizCamera
-from pyezviz.constants import DefenseModeType, DeviceCatagories
+from pyezviz.cas import EzvizCAS
+from pyezviz.constants import (DEFAULT_TIMEOUT, FEATURE_CODE, MAX_RETRIES,
+                               DefenseModeType, DeviceCatagories)
+from pyezviz.exceptions import HTTPError, InvalidURL, PyEzvizError
 
 API_ENDPOINT_CLOUDDEVICES = "/api/cloud/v2/cloudDevices/getAll"
 API_ENDPOINT_PAGELIST = "/v3/userdevices/v1/devices/pagelist"
@@ -15,22 +18,13 @@ API_ENDPOINT_REFRESH_SESSION_ID = "/v3/apigateway/login"
 API_ENDPOINT_SWITCH_STATUS = "/switchStatus"
 API_ENDPOINT_PTZCONTROL = "/ptzControl"
 API_ENDPOINT_ALARM_SOUND = "/alarm/sound"
-API_ENDPOINT_SET_DEFENCE = "/camera/cameraAction!enableDefence.action"  # Needs changing
 API_ENDPOINT_DETECTION_SENSIBILITY = "/api/device/configAlgorithm"
 API_ENDPOINT_DETECTION_SENSIBILITY_GET = "/api/device/queryAlgorithmConfig"
 API_ENDPOINT_ALARMINFO_GET = "/v3/alarms/v2/advanced"
 API_ENDPOINT_SET_DEFENCE_SCHEDULE = "/api/device/defence/plan2"
 API_ENDPOINT_SWITCH_DEFENCE_MODE = "/v3/userdevices/v1/group/switchDefenceMode"
 API_ENDPOINT_SWITCH_SOUND_ALARM = "/sendAlarm"
-
-DEFAULT_TIMEOUT = 25
-MAX_RETRIES = 3
-FEATURE_CODE = "c22cb01f8cb83351422d82fad59c8e4e"
-# token = {"sessionId": None, "refreshSessionId": None}
-
-
-class PyEzvizError(Exception):
-    """Handle exception."""
+API_ENDPOINT_SERVER_INFO = "/v3/configurations/system/info"
 
 
 class EzvizClient:
@@ -48,16 +42,20 @@ class EzvizClient:
         self.account = account
         self.password = password
         self._session = None
-        self._token = token or {"session_id": None, "rf_session_id": None}
+        self._token = token or {
+            "session_id": None,
+            "rf_session_id": None,
+            "username": None,
+            "api_url": url,
+        }
         self._timeout = timeout
-        self.api_uri = url
 
     def _login(self):
         """Login to Ezviz API."""
 
         # Region code to url.
-        if len(self.api_uri.split(".")) == 1:
-            self.api_uri = "apii" + self.api_uri + ".ezvizlife.com"
+        if len(self._token["api_url"].split(".")) == 1:
+            self._token["api_url"] = "apii" + self._token["api_url"] + ".ezvizlife.com"
 
         # Ezviz API sends md5 of password
         temp = hashlib.md5()
@@ -67,21 +65,32 @@ class EzvizClient:
             "account": self.account,
             "password": md5pass,
             "featureCode": FEATURE_CODE,
+            "msgType": "0",
+            "cuName": "SFRDIDEw",
         }
 
         try:
             req = self._session.post(
-                "https://" + self.api_uri + API_ENDPOINT_LOGIN,
+                "https://" + self._token["api_url"] + API_ENDPOINT_LOGIN,
                 allow_redirects=False,
-                headers={"clientType": "3"},
+                headers={
+                    "clientType": "3",
+                    "customno": "1000001",
+                    "clientNo": "web_site",
+                    "appId": "ys7",
+                    "lang": "en",
+                },
                 data=payload,
                 timeout=self._timeout,
             )
 
             req.raise_for_status()
 
+        except requests.ConnectionError as err:
+            raise InvalidURL("A Invalid URL or Proxy error occured") from err
+
         except requests.HTTPError as err:
-            raise requests.HTTPError(err)
+            raise HTTPError from err
 
         try:
             json_result = req.json()
@@ -95,9 +104,9 @@ class EzvizClient:
             ) from err
 
         if json_result["meta"]["code"] == 1100:
-            self.api_uri = json_result["loginArea"]["apiDomain"]
+            self._token["api_url"] = json_result["loginArea"]["apiDomain"]
             print("Region incorrect!")
-            print(f"Your region url: {self.api_uri}")
+            print(f"Your region url: {self._token['api_url']}")
             self._session = None
             return self.login()
 
@@ -112,13 +121,68 @@ class EzvizClient:
 
         self._token["session_id"] = str(json_result["loginSession"]["sessionId"])
         self._token["rf_session_id"] = str(json_result["loginSession"]["rfSessionId"])
+        self._token["username"] = str(json_result["loginUser"]["username"])
+        self._token["api_url"] = str(json_result["loginArea"]["apiDomain"])
         if not self._token["session_id"]:
             raise PyEzvizError(
                 f"Login error: Please check your username/password: {req.text}"
             )
 
+        self._token["service_urls"] = self.get_service_urls()
+
         return self._token
 
+    def get_service_urls(self, max_retries=0):
+        """Get Ezviz service urls."""
+        if max_retries > MAX_RETRIES:
+            raise PyEzvizError("Can't gather proper data. Max retries exceeded.")
+
+        try:
+            req = self._session.get(
+                f"https://{self._token['api_url']}{API_ENDPOINT_SERVER_INFO}",
+                headers={
+                    "sessionId": self._token["session_id"],
+                    "featureCode": FEATURE_CODE,
+                },
+                timeout=self._timeout,
+            )
+
+            req.raise_for_status()
+
+        except requests.ConnectionError as err:
+            raise InvalidURL("A Invalid URL or Proxy error occured") from err
+
+        except requests.HTTPError as err:
+            if err.response.status_code == 401:
+                # session is wrong, need to relogin
+                self.login()
+
+            raise HTTPError from err
+
+        if not req.text:
+            raise PyEzvizError("No data")
+
+        try:
+            json_output = req.json()
+
+        except ValueError as err:
+            raise PyEzvizError(
+                "Impossible to decode response: "
+                + str(err)
+                + "\nResponse was: "
+                + str(req.text)
+            ) from err
+
+        if json_output.get("meta").get("code") != 200:
+            logging.info(
+                "Json request error, relogging (max retries: %s)", str(max_retries)
+            )
+
+        service_urls = json_output["systemConfigInfo"]
+        service_urls["sysConf"] = service_urls["sysConf"].split("|")
+
+        return service_urls
+
     def _api_get_pagelist(self, page_filter=None, json_key=None, max_retries=0):
         """Get data from pagelist API."""
 
@@ -130,7 +194,7 @@ class EzvizClient:
 
         try:
             req = self._session.get(
-                "https://" + self.api_uri + API_ENDPOINT_PAGELIST,
+                "https://" + self._token["api_url"] + API_ENDPOINT_PAGELIST,
                 headers={"sessionId": self._token["session_id"]},
                 params={"filter": page_filter},
                 timeout=self._timeout,
@@ -144,7 +208,7 @@ class EzvizClient:
                 self.login()
                 return self._api_get_pagelist(page_filter, json_key, max_retries + 1)
 
-            raise requests.HTTPError(err)
+            raise HTTPError from err
 
         if not req.text:
             raise PyEzvizError("No data")
@@ -191,7 +255,7 @@ class EzvizClient:
 
         try:
             req = self._session.get(
-                "https://" + self.api_uri + API_ENDPOINT_ALARMINFO_GET,
+                "https://" + self._token["api_url"] + API_ENDPOINT_ALARMINFO_GET,
                 headers={"sessionId": self._token["session_id"]},
                 params={
                     "deviceSerials": serial,
@@ -210,7 +274,7 @@ class EzvizClient:
                 self.login()
                 return self.get_alarminfo(serial, max_retries + 1)
 
-            raise requests.HTTPError(err)
+            raise HTTPError from err
 
         if req.text == "":
             raise PyEzvizError("No data")
@@ -236,7 +300,7 @@ class EzvizClient:
         try:
             req = self._session.put(
                 "https://"
-                + self.api_uri
+                + self._token["api_url"]
                 + API_ENDPOINT_DEVICES
                 + serial
                 + "/1/1/"
@@ -260,7 +324,7 @@ class EzvizClient:
                 self.login()
                 return self._switch_status(serial, status_type, enable, max_retries + 1)
 
-            raise requests.HTTPError(err)
+            raise HTTPError from err
 
         try:
             json_output = req.json()
@@ -288,7 +352,7 @@ class EzvizClient:
         try:
             req = self._session.put(
                 "https://"
-                + self.api_uri
+                + self._token["api_url"]
                 + API_ENDPOINT_DEVICES
                 + serial
                 + "/0"
@@ -308,7 +372,7 @@ class EzvizClient:
                 self.login()
                 return self.sound_alarm(serial, enable, max_retries + 1)
 
-            raise requests.HTTPError(err)
+            raise HTTPError from err
 
         try:
             json_output = req.json()
@@ -465,7 +529,7 @@ class EzvizClient:
         try:
             req = self._session.put(
                 "https://"
-                + self.api_uri
+                + self._token["api_url"]
                 + API_ENDPOINT_DEVICES
                 + serial
                 + API_ENDPOINT_PTZCONTROL,
@@ -484,7 +548,7 @@ class EzvizClient:
             req.raise_for_status()
 
         except requests.HTTPError as err:
-            raise requests.HTTPError(err)
+            raise HTTPError from err
 
         return req.text
 
@@ -492,12 +556,16 @@ class EzvizClient:
         """Set http session."""
         if self._session is None:
             self._session = requests.session()
-            return self._login()
+            self._session.headers.update(
+                {"User-Agent": "okhttp/3.12.1"}
+            )  # Android generic user agent.
 
         if self._token["session_id"] and self._token["rf_session_id"]:
             try:
                 req = self._session.put(
-                    "https://" + self.api_uri + API_ENDPOINT_REFRESH_SESSION_ID,
+                    "https://"
+                    + self._token["api_url"]
+                    + API_ENDPOINT_REFRESH_SESSION_ID,
                     data={
                         "refreshSessionId": self._token["rf_session_id"],
                         "featureCode": FEATURE_CODE,
@@ -508,7 +576,7 @@ class EzvizClient:
                 req.raise_for_status()
 
             except requests.HTTPError as err:
-                raise requests.HTTPError(err)
+                raise HTTPError from err
 
             try:
                 json_result = req.json()
@@ -528,53 +596,17 @@ class EzvizClient:
             if not self._token["session_id"]:
                 raise PyEzvizError(f"Relogin required: {req.text}")
 
+            if not self._token["service_urls"]:
+                self._token["service_urls"] = self.get_service_urls()
+
             return self._token
 
-        return True
+        return self._login()
 
-    def data_report(self, serial, enable=1, max_retries=0):
-        """Enable alarm notifications."""
-        if max_retries > MAX_RETRIES:
-            raise PyEzvizError("Can't gather proper data. Max retries exceeded.")
-
-        try:
-            req = self._session.post(
-                "https://" + self.api_uri + API_ENDPOINT_SET_DEFENCE,
-                headers={"sessionId": self._token["session_id"]},
-                data={
-                    "deviceSerial": serial,
-                    "defenceType": "Global",
-                    "enablePlan": enable,
-                    "channelNo": "1",
-                },
-                timeout=self._timeout,
-            )
-
-            req.raise_for_status()
-
-        except requests.HTTPError as err:
-            if err.response.status_code == 401:
-                # session is wrong, need to relogin
-                self.login()
-                return self.data_report(serial, enable, max_retries + 1)
-
-            raise requests.HTTPError(err)
-
-        try:
-            json_output = req.json()
-
-        except ValueError as err:
-            raise PyEzvizError(
-                "Impossible to decode response: "
-                + str(err)
-                + "\nResponse was: "
-                + str(req.text)
-            ) from err
-
-        if json_output.get("resultCode") != "0":
-            raise PyEzvizError(
-                f"Could not set alarm notification: Got {req.status_code} : {req.text})"
-            )
+    def set_camera_defence(self, serial, enable):
+        """Enable/Disable motion detection on camera."""
+        cas_client = EzvizCAS(self._token)
+        cas_client.set_camera_defence_state(serial, enable)
 
         return True
 
@@ -594,7 +626,7 @@ class EzvizClient:
         )
         try:
             req = self._session.post(
-                "https://" + self.api_uri + API_ENDPOINT_SET_DEFENCE_SCHEDULE,
+                "https://" + self._token["api_url"] + API_ENDPOINT_SET_DEFENCE_SCHEDULE,
                 headers={"sessionId": self._token["session_id"]},
                 data={
                     "devTimingPlan": schedulestring,
@@ -612,7 +644,7 @@ class EzvizClient:
                     serial, schedule, enable, max_retries + 1
                 )
 
-            raise requests.HTTPError(err)
+            raise HTTPError from err
 
         try:
             json_output = req.json()
@@ -639,7 +671,7 @@ class EzvizClient:
 
         try:
             req = self._session.post(
-                "https://" + self.api_uri + API_ENDPOINT_SWITCH_DEFENCE_MODE,
+                "https://" + self._token["api_url"] + API_ENDPOINT_SWITCH_DEFENCE_MODE,
                 headers={"sessionId": self._token["session_id"]},
                 data={
                     "groupId": -1,
@@ -656,7 +688,7 @@ class EzvizClient:
                 self.login()
                 return self.api_set_defence_mode(mode, max_retries + 1)
 
-            raise requests.HTTPError(err)
+            raise HTTPError from err
 
         try:
             json_output = req.json()
@@ -688,7 +720,9 @@ class EzvizClient:
 
         try:
             req = self._session.post(
-                "https://" + self.api_uri + API_ENDPOINT_DETECTION_SENSIBILITY,
+                "https://"
+                + self._token["api_url"]
+                + API_ENDPOINT_DETECTION_SENSIBILITY,
                 headers={"sessionId": self._token["session_id"]},
                 data={
                     "subSerial": serial,
@@ -709,7 +743,7 @@ class EzvizClient:
                     serial, sensibility, type_value, max_retries + 1
                 )
 
-            raise requests.HTTPError(err)
+            raise HTTPError from err
 
         try:
             response_json = req.json()
@@ -729,7 +763,9 @@ class EzvizClient:
 
         try:
             req = self._session.post(
-                "https://" + self.api_uri + API_ENDPOINT_DETECTION_SENSIBILITY_GET,
+                "https://"
+                + self._token["api_url"]
+                + API_ENDPOINT_DETECTION_SENSIBILITY_GET,
                 headers={"sessionId": self._token["session_id"]},
                 data={
                     "subSerial": serial,
@@ -747,7 +783,7 @@ class EzvizClient:
                     serial, type_value, max_retries + 1
                 )
 
-            raise requests.HTTPError(err)
+            raise HTTPError from err
 
         try:
             response_json = req.json()
@@ -779,7 +815,7 @@ class EzvizClient:
         try:
             req = self._session.put(
                 "https://"
-                + self.api_uri
+                + self._token["api_url"]
                 + API_ENDPOINT_DEVICES
                 + serial
                 + API_ENDPOINT_ALARM_SOUND,
@@ -801,7 +837,7 @@ class EzvizClient:
                 self.login()
                 return self.alarm_sound(serial, sound_type, enable, max_retries + 1)
 
-            raise requests.HTTPError(err)
+            raise HTTPError from err
 
         return True
 
